# (C) 2004-2006 Veit Wahlich <cru |at| zodia |dot| de>
# Released under the terms of the General Public License.
# Part of the Perl Script Pages. See POD for more information.

package PSP::CGI;
our $VERSION='0.7';

use warnings;
use strict;
use POSIX;
use PSP::CGI::Info;
use PSP::CGI::Transform;
use PSP::CGI::Generate;

=head1 NAME

PSP::CGI - CGI class for the Perl Script Pages


=head1 SYNOPSIS

	use PSP::CGI;
	my $cgi = new PSP::CGI();
	
	my $value = $cgi->param($key);
	my @values = $cgi->multiparam($key);
	
	my $param = $cgi->param();
	my $value = $param->{$key};
	
	my @keys = $cgi->params();
	
	my $upload_item = $cgi->upload($key);
	
	my $upload = $cgi->upload();
	my $upload_item = $upload->{$key};
	
	my $filename = $upload_item->{filename};
	my $size = $upload_item->{size};
	my $content = $upload_item->{content};
	
	my @keys = $cgi->uploads();
	
	my $value = $cgi->cookie($key);
	
	my $cookie = $cgi->cookie();
	my $value = $cookie->{$key};
	
	my @keys = $cgi->cookies();
	
	my $success = $cgi->setcookie($key, $value, $lifetime,
  		domain => $domain,
			path   => $path,
			secure => $bool
		);
	
	$cgi->setheader(
			HeaderOne => 'foo',
			HeaderTwo => 'bar'
		);
	
	my $text = $cgi->flushHeaders();


=head1 DESCRIPTION

This module provides object oriented CGI capabilities for the
Perl Script Pages (PSP) embedded Perl engine.
See L<PSP::Engine> for more information.


=head1 OVERVIEW


=head2 Object Constructors

=over

=item new

Reads GET and POST variables (CGI variables and file uploads) and Cookies,
creates a new object of the PSP::CGI class.
The new constructor is the only object constructor for this module.
See L</"new object constructor"> for details.

=back


=head2 Object Methods

=over

=item param

Returns either the valie of a named CGI parameter or a reference to the
internal parameter hash.
See L</"param method"> for details.

=item multiparam

Returns the values of a named CGI parameter as an array, allowing direct access
to parameters that were transmitted twice or more (i.e. from form elements with
"multiple option" set).
See L</"multiparam method"> for details.

=item params

Returns an array of CGI parameter names supplied by the web browser.
See L</"params method"> for details.

=item upload

Returns either an upload item for a named HTTP upload or a reference to the
internal uploads hash.
Upload items are hashs containing more information about the upload,
see syntax and $item description for L</"upload method"> for details.

=item uploads

Returns an array of HTTP file upload names supplied by the web browser.
See L</"uploads method"> for details.

=item cookie

Returns either the value of a named cookie or a reference to the internal
cookies hash.
See L</"cookie method"> for details.

=item cookies

Returns an array of cookie names supplied by the web browser.
See L</"cookies method"> for details.

=item setcookie

Places a cookie of given name and value for a defined lifetime on the HTTP
header queue. Defining domain, path or HTTPS restrictions is optional.
See L</"setcookie method"> for details.

=item setheader

Places one or more headers of given names and values on the HTTP header queue.
See L</"setheader method"> for details.

=item flushHeaders

Returns the object's headers and cookie data in HTTP headers format and resets the data storage.
See L</"flushHeaders method"> for details.

=back


=head1 SYNTAX


=head2 new Object Constructor

Reads GET and POST variables (CGI variables and file uploads) and Cookies,
creates a new object of the PSP::CGI class.

	use PSP::CGI;
	my $cgi = new PSP::CGI();

=head3 Required Parameters

None.

=head3 Optional Parameters

None.

=cut
sub new{
	my $class=shift;
	my $self={
			send_headers   => { 'Content-Type' => 'text/html'},
			send_cookies   => {},
			# FIXME: Make object configurable using PSP::Config
			max_param_size => 2*1024*1024
		};
	bless($self,$class);
	$self->readCookies();
	$self->readParams();
	return($self);
}


=head2 param Method

Returns either the value of a named CGI parameter or a reference to the
internal parameter hash.

	my $value = $cgi->param($key);
	
	my $param = $cgi->param();
	my $value = $param->{$key};

If a requested parameter was transmitted twice or more (i.e. for a multi
select input form), the values are returned separated by ASCII NUL (0x00)
characters. You convert these parameters to an array as follows:

	my @array = split(/\x00/, $cgi->param($keyOfMultiselectInput));

This is done automatically by the L</"multiparam method">. See below.

=head3 Required Parameters

None.

=head3 Optional Parameters

=over

=item $key

The name of the CGI parameter to evaluate.

=back

=head3 Return Values

=over

=item $value

The value of the named CGI parameter or undef if the named parameter did not
exists.

=item $param

Returned only when no $key parameter was supplied or was undef.
A reference to the internal parameter hash, allowing direct access to
CGI parameters.

=back

=cut
sub param{
	my $self=shift;
	my $key=shift;
	if(defined($key)){
		return($self->{param}->{$key});
	}
	else{
		return($self->{param});
	}
}


=head2 multiparam Method

Returns the values of a named CGI parameter as an array, allowing direct access
to parameters that were transmitted twice or more (i.e. from form elements with
"multiple option" set).

	my @values = $cgi->multiparam($key);

=head3 Required Parameters

=over

=item $key

The name of the CGI parameter to evaluate.

=back

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item @values

An array containing one element per value found for the named CGI parameter.
If the named parameter did not exist, an empty array is returned.

=back

=cut
sub multiparam{
	my $self=shift;
	my $key=shift;
	if(defined($key)){
		return(split(/\x00/,$self->{param}->{$key}));
	}
	else{
		return(());
	}
}


=head2 params Method

Returns an array of CGI parameter names supplied by the web browser.

	my @keys = $cgi->params();

=head3 Required Parameters

None.

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item @keys

Array of CGI parameter names/hash search keys.

=back

=cut
sub params{
	my $self=shift;
	return(keys(%{$self->{param}}));		
}


=head2 upload Method

Returns either an upload item for a named HTTP upload or a reference to the
internal uploads hash.
Upload items are hashs containing more information about the upload,
see syntax and $item description below.

	my $item = $cgi->upload($key);
	
	my $upload = $cgi->upload();
	my $item = $upload->{$key};
	
	my $filename = $item->{filename};
	my $size = $item->{size};
	my $content = $item->{content};

=head3 Required Parameters

None.

=head3 Optional Parameters

=over

=item $key

The name of the HTTP upload to evaluate.

=back

=head3 Return Values

=over

=item $item

A reference to a hash structure containing the following elements:
"filename", the name of the file uploaded;
"size", size of the uploaded data in bytes;
"content", the actual data uploaded.

=item $upload

Returned only when no $key parameter was supplied or was undef.
A reference to the internal uploads hash, allowing direct access to
HTTP uploads.

=back

=cut
sub upload{
	my $self=shift;
	my $key=shift;
	if(defined($key)){
		return($self->{upload}->{$key});
	}
	else{
		return($self->{upload});
	}
}


=head2 uploads Method

Returns an array of HTTP file upload names supplied by the web browser.

	my @keys = $cgi->uploads();

=head3 Required Parameters

None.

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item @keys

Array of HTTP file upload names/hash search keys.

=back

=cut
sub uploads{
	my $self=shift;
	return(keys(%{$self->{upload}}));		
}


=head2 cookie Method

Returns either the value of a named cookie or a reference to the
internal cookies hash.

	my $value = $cgi->cookie($key);
	
	my $cookie = $cgi->cookie();
	my $value = $cookie->{$key};

=head3 Required Parameters

None.

=head3 Optional Parameters

=over

=item $key

The name of the cookie to evaluate.

=back

=head3 Return Values

=over

=item $value

The value of the named cookie or undef if the named cookie did not
exist.

=item $cookie

Returned only when no $key parameter was supplied or was undef.
A reference to the internal cookies hash, allowing direct access to
cookies.

=back

=cut
sub cookie{
	my $self=shift;
	my $key=shift;
	if(defined($key)){
		return($self->{cookie}->{$key});
	}
	else{
		return($self->{cookie});
	}
}


=head2 cookies Method

Returns an array of cookie names supplied by the web browser.

	my @keys = $cgi->cookies();

=head3 Required Parameters

None.

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item @keys

Array of cookie names/hash search keys.

=back

=cut
sub cookies{
	my $self=shift;
	return(keys(%{$self->{cookie}}));		
}

=head2 setcookie Method

Places a cookie of given name and value for a defined lifetime on the HTTP
header queue.
Defining domain, path or HTTPS restrictions is optional.

	my $success = $cgi->setcookie($key, $value, $lifetime,
			domain => $domain,
			path   => $path,
			secure => $bool
		);

=head3 Required Parameters

=over

=item $key

The name of the cookie's key/value pair.
If a cookie of the same name was set before, it will get overwritten.

=item $value

The value of the cookie's key/value pair.

=item $lifetime

Lifetime of the cookie in seconds (How long the browser shall store the cookie).

=back

=head3 Optional Parameters

The following parameters are hash key/value parameters.

=over

=item domain

A domain/host name to restrict the cookie to.
By default the cookie will restricted to the server's hostname by the browser
if te domain parameter is omitted.

=item path

Path to restrict the cookie to.
By default the cookie will not be restricted to a path, allowing all files on
the host name to receive the cookie.

=item secure

Boolean parameter, tells the browser to submit the cookie only if HTTPS is used.
By default the cookie will be submitted regardless of the connection being
encrypted or not.

=back

=head3 Return Values

=over

=item $success

Returns 1 on success or undef on error.

=back

=cut
sub setcookie{
	my $self=shift;
	my $key=shift;
	my $value=shift;
	my $lifetime=shift;
	my %hash=@_;
	if(defined($self) && defined($key) && defined($lifetime)){
		$self->{send_cookies}->{encodeHttp($key)}=[encodeHttp($value),$lifetime,(exists($hash{domain}))?($hash{domain}):(''),(exists($hash{path}))?($hash{path}):('/'),($hash{secure} && 1 || 0)];
		return(1);
	}
	else{
		return(undef);
	}
}


=head2 setheader Method

Places one or more headers of given names and values on the HTTP header queue.

	$cgi->setheader(
			HeaderOne => 'foo',
			HeaderTwo => 'bar'
		);

If the a header value contains \r or \n characters, everything beyond the first
\r or \n will be removed. This prevents header injections.

=head3 Required Parameters

The method accepts a hash of one or more HTTP headers and corresponding
values. Existing headers of same name will be overwritten.
See example above.

=head3 Optional Parameters

None.

=head3 Return Values

None.

=cut
sub setheader{
	my $self=shift;
	my %hash=@_;
	foreach(keys(%hash)){
		$hash{$_}=~s/[\r\n].*$//sg;
		$self->{send_headers}->{$_}=$hash{$_};
	}
}


=head2 flushHeaders method

Returns the object's headers and cookie data in HTTP headers format and resets
the data storage.

	my $text = $cgi->flushHeaders();

=head3 Required Parameters

None.

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $text

A text string containing the headers and cookies set in HTTP headers format.

=back

=cut
sub flushHeaders{
	my $self=shift;
	my $headers='';
	foreach(keys(%{$self->{send_headers}})){
		$headers.=$_.': '.$self->{send_headers}->{$_}."\r\n";
	}
	foreach(keys(%{$self->{send_cookies}})){
		$headers.=generateCookie($_,$self->{send_cookies}->{$_});
	}
	$headers.="\r\n";
	$self->{send_headers}={'Content-Type' => 'text/html'};
	$self->{send_cookies}={};
	return($headers);
}


# Package function.
sub generateCookie{
	my $key=shift;
	my $data=shift;
  my $expires=strftime("%a, %d-%b-%Y %H:%M:%S GMT",gmtime(time+$data->[1]));
  my $cookie=sprintf(
			'Set-Cookie: %s=%s; expires=%s; max-age=%s; path=%s'.
			(($data->[2] ne '')?('; domain='.$data->[2]):('')).(($data->[4])?('; secure=1'):(''))."\r\n",
			$key,$data->[0],$expires,$data->[1],$data->[3]
		);
	return($cookie);
}


# Object method.
sub readParams{
	my $self=shift;
	my $formdata;
	if(exists($ENV{REQUEST_METHOD})){
		if($ENV{REQUEST_METHOD} eq 'GET' && exists($ENV{QUERY_STRING})){
			if(length($ENV{QUERY_STRING}) > $self->{max_param_size}){
				return({},{});
			}
			$self->{param}=decodeParams(\$ENV{QUERY_STRING});
			$self->{upload}={};
		}
		elsif($ENV{REQUEST_METHOD} eq 'POST' && exists($ENV{CONTENT_LENGTH})){
			read(STDIN,$formdata,$ENV{CONTENT_LENGTH});
			if(exists($ENV{CONTENT_TYPE}) && $ENV{CONTENT_TYPE}=~/^multipart\/form\-data\;(\s|)boundary\=(.*)[\r\n]{0,2}$/){
				($self->{param},$self->{upload})=decodeMultipart(\$formdata,$2);
			}
			else{
				$self->{param}=decodeParams(\$formdata);
				$self->{upload}={};
			}
		}
	}
	return({},{});
}


# Object method.
sub readCookies{
	my $self=shift;
	my $cookie={};
	my $name;
	my $value;
	if(exists($ENV{HTTP_COOKIE}) && defined($ENV{HTTP_COOKIE})){
		foreach(split(/\;[\s\t]{0,}/,$ENV{HTTP_COOKIE})){
			($name,$value)=split(/=/,$_,2);
			$name=decodeHttp($name);
			$cookie->{$name}=decodeHttp($value);
		}
	}
	$self->{cookie}=$cookie;
}


# Package function.
sub decodeMultipart{
	my $formdata=shift;
	my $boundary=shift;
	my $param={};
	my $upload={};
	my @data=split(/\-\-$boundary/s,${$formdata});
	my($header,$data,$name,$filename);
	foreach(@data){
		$name=undef;
		$filename=undef;
		if(($header,$data)=/^\r\n(.*?)\r\n\r\n(.*)\r\n$/s){
			foreach(split(/\r\n/,$header)){
				if(/^Content\-Disposition:\sform\-data\;\s+(.*)$/){
					foreach(split(/\;\s+/,$1)){
						if(/^name=\"([^\"]+)\"$/){
							$name=$1;
						}
						elsif(/^filename=\"([^\"]+)\"$/){
							$filename=$1;
						}
					}
				}
			}
			if(defined($filename)){
				$upload->{$name}={
						filename => $filename,
						content  => $data,
						size     => length($data)
					};
			}
			else{
				$param->{$name}=$data;
			}
		}
	}
	return($param,$upload);
}


# Package function.
sub decodeParams{
	my $formdata=shift;
	my $param={};
	my($name,$value);
	foreach(split(/\&/,${$formdata})){
		($name,$value)=split(/=/,$_,2);
		$name=decodeHttp($name);
		if(exists($param->{$name})){
			$param->{$name}.="\x00".decodeHttp($value);
		}
		else{
			$param->{$name}=decodeHttp($value);
		}
	}
	return($param);
}

1;


=head1 AUTHOR

Veit Wahlich

E-Mail: cru |at| zodia |dot| de

WWW: L<http://home.ircnet.de/cru/>


=head1 VERSION

v0.7
Wednesday, 18 January 2006


=head1 COPYRIGHT/LICENSE

Copyright 2004-2006 Veit Wahlich

This software is distributed as free (libre) software under the terms of the
GNU General Public License, version 2 E<lt>L<http://www.gnu.org/copyleft/gpl.html>E<gt>.
The author disclaims responsibility of any damage or harm caused directly or
indirectly by usage of this software. Use only at your own risk.
