# (C) 2004-2006 Veit Wahlich <cru |at| zodia |dot| de>
# Released under the terms of the General Public License.
# Part of the Perl Script Pages. See POD for more information.

package PSP::CGI::HTTP;
our $VERSION='0.7';

use warnings;
use strict;
use Exporter;
use PSP::CGI::Transform;

our(@ISA,@EXPORT_OK);
@ISA=qw(Exporter);
@EXPORT_OK=qw(redirect notFound status);

=head1 NAME

PSP::CGI::HTTP - exportable functions for generating HTTP status pages.


=head1 SYNOPSIS

	use PSP::CGI::HTTP;
	my $success;
	$success = PSP::CGI::HTTP::redirect($psp,$url,$code);
	$success = PSP::CGI::HTTP::notFound($psp,$url);
	$success = PSP::CGI::HTTP::status($psp,$url,$code,$codeName,$message,%extraHeaders);
	
	use PSP::CGI::HTTP qw(redirect notFound status);
	my $success;
	$success = redirect($psp,$url,$code);
	$success = notFound($psp,$url);
	$success = status($psp,$code,$codeName,$message,%extraHeaders);


=head1 DESCRIPTION

This module provides exportable functions that create HTTP status pages
commonly used in CGI programs i.e. for redirecting the browser to another
URL, telling the browser about missing files, etc..
This module was created as a companion to the Perl Script Pages Engine.
See L<PSP::Engine> for more information.

=head1 OVERVIEW


=head2 Exportable Functions

=over

=item redirect

Creates a HTTP redirect ("Moved") message, redirecting the web browser to
another URL. 
See L</"redirect function"> for details.

=item notFound

Creates a HTTP 404 ("Not Found") message, telling the web browser the
requested URL does not exist.
See L</"notFound function"> for details.

=item status

Creates a user defined HTTP status message.
See L</"status function"> for details.

=back


=head1 SYNTAX


=head2 redirect Function

Creates a HTTP redirect ("Moved") message, redirecting the web browser to
another URL. 

	use PSP::CGI::HTTP;
	my $success = PSP::CGI::HTTP::redirect($psp,$url,$code);
	
	use PSP::CGI::HTTP qw(redirect);
	my $success = redirect($psp,$url,$code);

Former content of the PSP output queue and CGI headers are flushed.

This function can be exported.

=head3 Required Parameters

=over

=item $psp

The L<PSP::Engine|PSP::Engine> object to flush and push the message to.
See L<PSP::Engine> for details.

=item $url

Text string describing the URL to redirect the web browser to.
Note that this variable is not sanitized for HTTP header output by this
function.

=back

=head3 Optional Parameters

=over

=item $code

HTTP status code to use. Might be either 301 ("Moved Permanently") or
302 ("Moved Temporarily"). By default, 302 ("Moved Temporarily") is used.

=back

=head3 Return Values

=over

=item $success

Returns 1 on success or undef on error.
An undef value also means the PSP output queue and CGI HTTP headers were NOT
flushed.

=back

=cut
sub redirect{
	my $psp=shift || return(undef);
	my $url=shift || return(undef);
	# 301: Moved Permanently, 302: Moved Temporarily
	my $code=shift || 302;
	return(status($psp,$code,'Moved','The document has moved <a href="'.htmlize($url).'">here</a>.','Location',$url));
}


=head2 notFound Function

Creates a HTTP 404 ("Not Found") message, telling the web browser the
requested URL does not exist.

	use PSP::CGI::HTTP;
	my $success = PSP::CGI::HTTP::notFound($psp,$url);
	
	use PSP::CGI::HTTP qw(notFound);
	my $success = notFound($psp,$url);

Former content of the PSP output queue and CGI headers are flushed.

This function can be exported.

=head3 Required Parameters

=over

=item $psp

The L<PSP::Engine|PSP::Engine> object to flush and push the message to.
See L<PSP::Engine> for details.

=item $url

Text string describing the URL that was not found.

=back

=head3 Optional Parameters

None.

=head3 Return Values

=over

=item $success

Returns 1 on success or undef on error.
An undef value also means the PSP output queue and CGI HTTP headers were NOT
flushed.

=back

=cut
sub notFound{
	my $psp=shift || return(undef);
	my $url=shift || return(undef);
	return(status($psp,404,'Not Found','The requested URL '.htmlize($url).' was not found on this server.'));
}


=head2 status Function

Creates a user defined HTTP status message.

	use PSP::CGI::HTTP;
	my $success = PSP::CGI::HTTP::status($psp,$url,$code,$codeName,$message,%extraHeaders);
	
	use PSP::CGI::HTTP qw(status);
	my $success = status($psp,$code,$codeName,$message,%extraHeaders);

Former content of the PSP output queue and CGI headers are flushed.

This function can be exported.

=head3 Required Parameters

=over

=item $psp

The L<PSP::Engine|PSP::Engine> object to flush and push the message to.
See L<PSP::Engine> for details.

=item $codeName

"Name" of the status, i.e. "Not Found" for code 404.

=item $message

The HTML message to show to the user, describing the status.

=back

=head3 Optional Parameters

=over

=item %extraHeaders

A hash containing additional headers to send to the client.

=back

=head3 Return Values

=over

=item $success

Returns 1 on success or undef on error.
An undef value also means the PSP output queue and CGI HTTP headers were NOT
flushed.

=back

=cut
sub status{
	my $psp=shift || return(undef);
	my $code=shift || return(undef);
	my $codeName=shift || return(undef);
	my $message=shift || return(undef);
	my %extraHeaders=@_;
	$psp->cgi->flushHeaders();
	$psp->flush();
	$psp->cgi->setheader(
			'Status'       => $code.' '.$codeName,
			'Content-Type' => 'text/html',
			%extraHeaders
		);
	$psp->print(<<__EOF);
<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<html>
	<head>
		<title>$code $codeName</title>
	</head>
	<body>
		<h1>$codeName</h1>
		<p>
			$message
		</p>
		<hr>
		<p>
			Perl Server Pages Engine - (c)2004-2006 Veit Wahlich &lt;cru [<strike>at</strike>] zodia [<strike>dot</strike>] de&gt;
		</p>
	</body>
</html>
__EOF
}

1;


=head1 AUTHOR

Veit Wahlich

E-Mail: cru |at| zodia |dot| de

WWW: L<http://home.ircnet.de/cru/>


=head1 VERSION

v0.7
Wednesday, 18 January 2006


=head1 COPYRIGHT/LICENSE

Copyright 2004-2006 Veit Wahlich

This software is distributed as free (libre) software under the terms of the
GNU General Public License, version 2 E<lt>L<http://www.gnu.org/copyleft/gpl.html>E<gt>.
The author disclaims responsibility of any damage or harm caused directly or
indirectly by usage of this software. Use only at your own risk.
